var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from "react";
import createEmotionServer from "@emotion/server/create-instance";
import { CacheProvider as DefaultCacheProvider } from "@emotion/react";
import createCache from "@emotion/cache";
import DefaultDocument from "next/document";
/** @see <https://docs.tss-react.dev/ssr/next> */
export function createEmotionSsrAdvancedApproach(options, CacheProvider = DefaultCacheProvider) {
    let cache = undefined;
    const createSpecificCacheCache = () => (cache = createCache(options));
    function augmentDocumentWithEmotionCache(Document) {
        var _a, _b;
        const super_getInitialProps = (_b = (_a = Document.getInitialProps) === null || _a === void 0 ? void 0 : _a.bind(Document)) !== null && _b !== void 0 ? _b : DefaultDocument.getInitialProps.bind(DefaultDocument);
        Document.getInitialProps = (appContext) => __awaiter(this, void 0, void 0, function* () {
            const cache = createSpecificCacheCache();
            const emotionServer = createEmotionServer(cache);
            const initialProps = yield super_getInitialProps(appContext);
            const emotionStyles = emotionServer
                .extractCriticalToChunks(initialProps.html)
                .styles.filter(({ css }) => css !== "")
                .map(style => (React.createElement("style", { "data-emotion": `${style.key} ${style.ids.join(" ")}`, key: style.key, dangerouslySetInnerHTML: { "__html": style.css } })));
            const otherStyles = React.Children.toArray(initialProps.styles);
            return Object.assign(Object.assign({}, initialProps), { "styles": !!cache.sheet.prepend
                    ? [...emotionStyles, ...otherStyles]
                    : [...otherStyles, ...emotionStyles] });
        });
    }
    function withAppEmotionCache(App) {
        function AppWithEmotionCache(props) {
            return (React.createElement(CacheProvider, { value: cache !== null && cache !== void 0 ? cache : createSpecificCacheCache() },
                React.createElement(App, Object.assign({}, props))));
        }
        Object.keys(App).forEach(staticMethod => (AppWithEmotionCache[staticMethod] = App[staticMethod]));
        AppWithEmotionCache.displayName = AppWithEmotionCache.name;
        return AppWithEmotionCache;
    }
    return { withAppEmotionCache, augmentDocumentWithEmotionCache };
}
